const fs = require('fs')
const readline = require('readline')
const { google } = require('googleapis')


class Worksheet {
   

    constructor(title) {
        this.type = '' 
        this.title = title
        this.values = null
    }

    setType() {

        if (this.values && this.values[0] && this.values[0][0]) {
            let type = this.values[0][0]
            this.type =  ('' + type).toLowerCase().trim()            
        }
    }

}

class Workbook {


    constructor() {
        this.sheets = []
    }


    addSheet(title) {
            
        let sheet = new Worksheet(title)       

        this.sheets.push(sheet)

        return sheet
    }
}


class GoogleApi {

    constructor(spreadsheetId) {
        this.SCOPES = ['https://www.googleapis.com/auth/spreadsheets.readonly'];
        this.TOKEN_PATH = 'token.json';
        this.auth = null
        this.spreadsheetId = spreadsheetId
    }


    init() {

        let me = this

        let promise = new Promise(function (res, rej) {

            // Load client secrets from a local file.
            fs.readFile('credentials.json', (err, content) => {
                if (err) return console.log('Error loading client secret file:', err);
                // Authorize a client with credentials, then call the Google Sheets API.
                me.authorize(JSON.parse(content), res);  // this.listMajors

                //res()
                //promise.resolve()
            });

        })

        return promise;
    }



    /**
     * Create an OAuth2 client with the given credentials, and then execute the
     * given callback function.
     * @param {Object} credentials The authorization client credentials.
     * @param {function} callback The callback to call with the authorized client.
     */
    authorize(credentials, callback) {
        //  let me = this
        const { client_secret, client_id, redirect_uris } = credentials.installed;
        this.auth = new google.auth.OAuth2(
            client_id, client_secret, redirect_uris[0]);

        // Check if we have previously stored a token.
        fs.readFile(this.TOKEN_PATH, (err, token) => {
            if (err) return this.getNewToken(callback);
            this.auth.setCredentials(JSON.parse(token));

            if (callback)
                callback(this.auth);

        });
    }

    /**
     * Get and store new token after prompting for user authorization, and then
     * execute the given callback with the authorized OAuth2 client.
     * @param {google.auth.OAuth2} oAuth2Client The OAuth2 client to get token for.
     * @param {getEventsCallback} callback The callback for the authorized client.
     */
    getNewToken(callback) {
        const authUrl = this.auth.generateAuthUrl({
            access_type: 'offline',
            scope: this.SCOPES,
        });
        console.log('Authorize this app by visiting this url:', authUrl);
        const rl = readline.createInterface({
            input: process.stdin,
            output: process.stdout,
        });
        rl.question('Enter the code from that page here: ', (code) => {
            rl.close();
            this.auth.getToken(code, (err, token) => {
                if (err) return console.error('Error while trying to retrieve access token', err);
                this.auth.setCredentials(token);
                // Store the token to disk for later program executions
                fs.writeFile(this.TOKEN_PATH, JSON.stringify(token), (err) => {
                    if (err) console.error(err);
                    console.log('Token stored to', this.TOKEN_PATH);
                });

                if (callback)
                    callback(this.auth);
            });
        });
    }

    getSheetInfo(spreadsheetId) {

        if (spreadsheetId)
            this.spreadsheetId = spreadsheetId

        let me = this

        let promise = new Promise(function (resolve, reject) {

            let auth = me.auth
            const sheets = google.sheets({ version: 'v4', auth });

            sheets.spreadsheets.get({
                spreadsheetId: me.spreadsheetId,   // '1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms',
                // range: '',  // !A2:B
            }, (err, res) => {

                if (err) reject('The API returned an error: ' + err);

                console.log(res.data.sheets[0].properties.title)

                resolve(res.data)

            });
        })

        return promise;
    }


    getAllSheets(spreadsheetId) {



        let workbook = new Workbook()


        let prom = this.getSheetInfo(spreadsheetId).then(res => {

            let promises = []


            res.sheets.forEach(sheet => {

                var title = sheet.properties.title
                var worksheet = workbook.addSheet(title)

                /*
                let i = ''

                i.startsWith
*/

                if (title.startsWith('#'))
                    return


                let sheetProm = this.getSheet(title).then(res => {

                    worksheet.values = res

                    worksheet.setType()

                })

                promises.push(sheetProm)

                console.log(res)


                console.log(title)


            })

            return Promise.all(promises).then(res => {
                return workbook
            })

        })

        // promises.push(prom)

        return prom
    }

    getSheet(range) {

        let me = this

        let promise = new Promise(function (resolve, reject) {

            let auth = me.auth
            const sheets = google.sheets({ version: 'v4', auth });

            sheets.spreadsheets.values.get({
                spreadsheetId: me.spreadsheetId,   // '1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms',
                range: range,  // !A2:B
            }, (err, res) => {

                if (err) reject('The API returned an error: ' + err);

                resolve(res.data.values)

            });
        })

        return promise;

    }

/*
    listMajors() {

        let auth = this.auth
        const sheets = google.sheets({ version: 'v4', auth });

        sheets.spreadsheets.values.get({
            spreadsheetId: '11mYefsEM7dI-6FSQfJIryq312HT7UbkgV559SacV3Q8',   // '1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms',
            range: 'Sheet1',  // !A2:B
        }, (err, res) => {
            if (err) return console.log('The API returned an error: ' + err);
            const rows = res.data.values;
            if (rows.length) {
                console.log('Name, Major:');
                // Print columns A and E, which correspond to indices 0 and 4.
                rows.map((row) => {
                    console.log(`${row[0]}, ${row[1]}`);
                });
            } else {
                console.log('No data found.');
            }
        });

    } */
}


// A basic test
// ============

/*
let googleApi = new GoogleApi('1qYuWxqGiAA0SUoH__3elKD_IgDZve55Zu782hJ38Qrc')  // 1MM74fTQB9fo1jI7FrU-oFwEL4CPH-m55IlO6eDkiXL0


googleApi.init().then(res => {
    console.log('ok, initialised ... ')

    return googleApi.getAllSheets().then(workbook => {

        console.log(workbook)

    })
})
*/


module.exports.Worksheet = Worksheet
module.exports.Workbook = Workbook
module.exports.GoogleApi = GoogleApi
